@@ -308,13 +308,17 @@ def draw_marker(surface, node, position="mid"):
                 scale_x, scale_y, translate_x, translate_y = \
                     preserve_ratio(surface, marker_node)
 
-                width, height, viewbox = node_format(surface, marker_node)
+                viewbox = node_format(surface, marker_node)[2]
+                width = base_scale * size(
+                    surface, node.get("markerWidth", "3"), "x")
+                height = base_scale * size(
+                    surface, node.get("markerHeight", "3"), "y")
                 if viewbox:
                     viewbox_width = viewbox[2]
                     viewbox_height = viewbox[3]
                 else:
-                    viewbox_width = width or 0
-                    viewbox_height = height or 0
+                    viewbox_width = width
+                    viewbox_height = height
 
                 surface.context.new_path()
                 for child in marker_node.children:
@@ -322,8 +326,8 @@ def draw_marker(surface, node, position="mid"):
                     surface.context.translate(current_x, current_y)
                     surface.context.rotate(angle)
                     surface.context.scale(
-                        base_scale / viewbox_width * float(scale_x),
-                        base_scale / viewbox_height * float(scale_y))
+                        width / viewbox_width * float(scale_x),
+                        height / viewbox_height * float(scale_y))
                     surface.context.translate(translate_x, translate_y)
                     surface.draw(child)
                     surface.context.restore()
@@ -333,12 +337,10 @@ def draw_marker(surface, node, position="mid"):
         node.pending_markers.append(pending_marker)
 
 
-def apply_filter_before(surface, node):
+def prepare_filter(surface, node, name):
     if node["id"] in surface.masks:
         return
 
-    names = urls(node.get("filter"))
-    name = names[0][1:] if names else None
     if name in surface.filters:
         filter_node = surface.filters[name]
         for child in filter_node.children:
@@ -355,14 +357,10 @@ def apply_filter_before(surface, node):
                 surface.context.translate(dx, dy)
 
 
-def apply_filter_after(surface, node):
-    surface.context.set_operator(BLEND_OPERATORS["normal"])
-
+def apply_filter_before_painting(surface, node, name):
     if node["id"] in surface.masks:
         return
 
-    names = urls(node.get("filter"))
-    name = names[0][1:] if names else None
     if name in surface.filters:
         filter_node = surface.filters[name]
         for child in filter_node.children:
@@ -370,8 +368,18 @@ def apply_filter_after(surface, node):
             if child.tag == "feBlend":
                 surface.context.set_operator(BLEND_OPERATORS.get(
                     child.get("mode", "normal"), BLEND_OPERATORS["normal"]))
+
+
+def apply_filter_after_painting(surface, node, name):
+    if node["id"] in surface.masks:
+        return
+
+    if name in surface.filters:
+        filter_node = surface.filters[name]
+        for child in filter_node.children:
             # Flood
-            elif child.tag == "feFlood":
+            if child.tag == "feFlood":
+                surface.context.save()
                 surface.context.new_path()
                 if filter_node.get("primitiveUnits") == "objectBoundingBox":
                     x = size(surface, node.get("x"), "x")
@@ -389,7 +397,7 @@ def apply_filter_after(surface, node):
                     paint(child.get("flood-color"))[1],
                     float(child.get("flood-opacity", 1))))
                 surface.context.fill()
-                surface.context.new_path()
+                surface.context.restore()
 
 
 def use(surface, node):
